# Generated by Django 3.2.12 on 2022-07-23 12:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('notification_system', '0003_auto_20220716_0639'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationevent',
            name='notification_event_type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Email'), (2, 'Internal message')], default=1, verbose_name='type'),
        ),
        migrations.AlterField(
            model_name='notificationevent',
            name='smtp_provider',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.smtpprovider', verbose_name='smtp provider'),
        ),
        migrations.AlterField(
            model_name='outgoingmessage',
            name='outgoing_message_type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Email'), (2, 'Internal message')], default=1, verbose_name='type'),
        ),
    ]
