# Generated by Django 3.2.12 on 2022-07-13 11:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import notification_system.validators


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('notification_system', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationevent',
            name='recipients',
            field=models.ManyToManyField(blank=True, related_name='recipients', to=settings.AUTH_USER_MODEL, verbose_name='members'),
        ),
        migrations.AlterField(
            model_name='notificationevent',
            name='notification_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='notification_events', to='notification_system.notificationgroup', verbose_name='notification group'),
        ),
        migrations.CreateModel(
            name='Media',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created time')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='updated time')),
                ('file', models.FileField(upload_to='notification_system/media/file/', validators=[notification_system.validators.validate_file_extension, notification_system.validators.validate_file_size], verbose_name='file')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='media', to=settings.AUTH_USER_MODEL, verbose_name='owner')),
            ],
            options={
                'verbose_name': 'Media',
                'verbose_name_plural': 'Media',
            },
        ),
    ]
