# Generated by Django 3.2.12 on 2022-07-16 06:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notification_system', '0002_auto_20220713_1151'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='notificationevent',
            name='notification_group',
        ),
        migrations.AddField(
            model_name='notificationevent',
            name='notification_groups',
            field=models.ManyToManyField(blank=True, related_name='notification_events', to='notification_system.NotificationGroup', verbose_name='notification groups'),
        ),
    ]
